@@ -33,29 +33,27 @@
 #' Compare Strings with or without Collation
 #'
 #' @description
-#' These functions  may be used to determine if two strings
-#' are equal, canonically equivalent (this is performed in a way more clever
+#' These functions may be used to determine if two strings
+#' are equal, canonically equivalent (this is performed in a much more clever
 #' fashion than when testing for equality),
 #' or to check whether they appear in
 #' a specific lexicographic order.
 #'
 #'
 #' @details
-#' All the functions are vectorized over \code{e1} and \code{e2}.
+#' All the functions listed here are vectorized over \code{e1} and \code{e2}.
 #'
 #' \code{stri_cmp_eq} tests whether two corresponding strings
 #' consist of exactly the same code points, while \code{stri_cmp_neq} allow
-#' to check whether there is any difference between them.
-#' These are locale-independent operations: for
-#' natural language text processing, in which the notion of canonical equivalence
-#' is more valid, this may not be exactly what
-#' you are looking for, see Examples.
-#' Also note that \pkg{stringi} always silently removes UTF-8
-#' BOMs from input strings,
-#' so e.g. \code{stri_cmp_eq} does not take BOMs into account while
-#' comparing strings.
+#' to check whether there is any difference between them. These are
+#' locale-independent operations: for natural language text processing, 
+#' in which the notion of canonical equivalence is more valid, this might
+#' not be exactly what you are looking for, see Examples.
+#' By the way, note that \pkg{stringi} always silently removes UTF-8
+#' BOMs from input strings, so e.g. \code{stri_cmp_eq} does not take
+#' BOMs into account while comparing strings.
 #'
-#' On the other hand, \code{stri_cmp_equiv} test for
+#' On the other hand, \code{stri_cmp_equiv} tests for
 #' canonical equivalence of two strings and is locale-dependent.
 #' Additionally, the \pkg{ICU}'s Collator may be tuned up so that
 #' e.g. the comparison is case-insensitive.
@@ -65,9 +63,8 @@
 #' What is more,  \code{stri_cmp_le} tests whether
 #' the elements in the first vector are less than or equal to
 #' the corresponding elements in the second vector,
-#' \code{stri_cmp_ge} whether they are greater or equal,
-#' \code{stri_cmp_lt} if less,
-#' and \code{stri_cmp_gt} if greater,
+#' \code{stri_cmp_ge} tests whether they are greater or equal,
+#' \code{stri_cmp_lt} if less, and \code{stri_cmp_gt} if greater,
 #' see also e.g. \code{\link{\%s<\%}}.
 #'
 #' Finally, \code{stri_compare} is an alias to \code{stri_cmp}. They both
@@ -86,10 +83,10 @@
 #' @param opts_collator a named list with \pkg{ICU} Collator's options
 #' as generated with \code{\link{stri_opts_collator}}, \code{NULL}
 #' for default collation options.
+#' @param ... additional settings for \code{opts_collator}
 #'
 #' @return The \code{stri_cmp} and \code{stri_compare} functions
-#' return an integer vector
-#' with comparison results of corresponding
+#' return an integer vector with comparison results of corresponding
 #' pairs of elements in \code{e1} and \code{e2}:
 #' \code{-1} if \code{e1[...] < e2[...]},
 #' \code{0} if they are canonically equivalent, and \code{1} if greater.
@@ -99,37 +96,37 @@
 #' in \code{e1} and \code{e2}.
 #'
 #' @examples
-#' \donttest{
 #' # in Polish ch < h:
-#' stri_cmp_lt("hladny", "chladny", stri_opts_collator(locale="pl_PL"))
+#' stri_cmp_lt("hladny", "chladny", locale="pl_PL")
 #'
 #' # in Slovak ch > h:
-#' stri_cmp_lt("hladny", "chladny", stri_opts_collator(locale="sk_SK"))
+#' stri_cmp_lt("hladny", "chladny", locale="sk_SK")
 #'
 #' # < or > (depends on locale):
 #' stri_cmp("hladny", "chladny")
 #'
 #' # ignore case differences:
-#' stri_cmp_equiv("hladny", "HLADNY", stri_opts_collator(strength=2))
+#' stri_cmp_equiv("hladny", "HLADNY", strength=2)
 #'
-#' # alse ignore diacritical differences:
-#' stri_cmp_equiv("hladn\u00FD", "hladny", stri_opts_collator(strength=1, locale="sk_SK"))
+#' # also ignore diacritical differences:
+#' stri_cmp_equiv("hladn\u00FD", "hladny", strength=1, locale="sk_SK")
 #'
-#' # non-normalized vs normalized string:
-#' stri_cmp_equiv(stri_trans_nfkd('\u0105'), '\u105')
+#' # non-Unicode-normalized vs normalized string:
+#' stri_cmp_equiv(stri_trans_nfkd("\u0105"), "\u105")
 #'
 #' # note the difference:
-#' stri_cmp_eq(stri_trans_nfkd('\u0105'), '\u105')
+#' stri_cmp_eq(stri_trans_nfkd("\u0105"), "\u105")
 #'
 #' # ligatures:
-#' stri_cmp_equiv("\ufb00", "ff", stri_opts_collator(strength=2))
-#' }
+#' stri_cmp_equiv("\ufb00", "ff", strength=2)
 #'
 #' @family locale_sensitive
 #' @export
 #' @rdname stri_compare
-stri_compare <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_integer", e1, e2, opts_collator, PACKAGE="stringi")
+stri_compare <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_integer, e1, e2, opts_collator)
 }
 
 
@@ -141,54 +138,66 @@ stri_cmp <- stri_compare
 #' @export
 #' @rdname stri_compare
 stri_cmp_eq <- function(e1, e2) {
-   .Call("stri_cmp_codepoints", e1, e2, 0L, PACKAGE="stringi")
+   .Call(C_stri_cmp_codepoints, e1, e2, 0L)
 }
 
 
 #' @export
 #' @rdname stri_compare
 stri_cmp_neq <- function(e1, e2) {
-   .Call("stri_cmp_codepoints", e1, e2, 1L, PACKAGE="stringi")
+   .Call(C_stri_cmp_codepoints, e1, e2, 1L)
 }
 
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_equiv <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(0L, 0L), PACKAGE="stringi")
+stri_cmp_equiv <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(0L, 0L))
 }
 
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_nequiv <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(0L, 1L), PACKAGE="stringi")
+stri_cmp_nequiv <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(0L, 1L))
 }
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_lt <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(-1L, 0L), PACKAGE="stringi")
+stri_cmp_lt <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(-1L, 0L))
 }
 
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_gt <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(1L, 0L), PACKAGE="stringi")
+stri_cmp_gt <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(1L, 0L))
 }
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_le <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(1L, 1L), PACKAGE="stringi")
+stri_cmp_le <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(1L, 1L))
 }
 
 
 #' @export
 #' @rdname stri_compare
-stri_cmp_ge <- function(e1, e2, opts_collator=NULL) {
-   .Call("stri_cmp_logical", e1, e2, opts_collator, c(-1L, 1L), PACKAGE="stringi")
+stri_cmp_ge <- function(e1, e2, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_cmp_logical, e1, e2, opts_collator, c(-1L, 1L))
 }
 
 
@@ -213,15 +222,13 @@ stri_cmp_ge <- function(e1, e2, opts_collator=NULL) {
 #' @param e1,e2 character vectors or objects coercible to character vectors
 #'
 #' @return All the functions return a logical vector
-#' indicating the result of the element-by-element comparison.
-#' As usual, the elements of shorter vectors are recycled as necessary.
+#' indicating the result of a pairwise comparison.
+#' As usual, the elements of shorter vectors are recycled if necessary.
 #'
 #'
 #' @examples
-#' \donttest{
 #' "a" %stri<% "b"
 #' c("a", "b", "c") %stri>=% "b"
-#' }
 #'
 #' @usage
 #' e1 \%s<\% e2
@@ -375,7 +382,7 @@ stri_cmp_ge <- function(e1, e2, opts_collator=NULL) {
 #'
 #' @description
 #' \link{stri_order} determines a permutation which rearranges
-#' strings into ascending or descending order.
+#' strings into an ascending or descending order.
 #' \link{stri_sort} sorts the vector according to a lexicographic order.
 #'
 #'
@@ -394,13 +401,14 @@ stri_cmp_ge <- function(e1, e2, opts_collator=NULL) {
 #' @param decreasing single logical value; should the sort order
 #'    be nondecreasing (\code{FALSE}, default)
 #'    or nonincreasing (\code{TRUE})?
-#' @param na_last single logical value controlling the treatment of \code{NA}s
+#' @param na_last single logical value; controls the treatment of \code{NA}s
 #'    in \code{str}. If \code{TRUE}, then missing values in \code{str} are put
-#'    at the end; if \code{FALSE}, they are put first;
+#'    at the end; if \code{FALSE}, they are put at the beginning;
 #'    if \code{NA}, then they are removed from the output.
 #' @param opts_collator a named list with \pkg{ICU} Collator's options
 #' as generated with \code{\link{stri_opts_collator}}, \code{NULL}
 #' for default collation options
+#' @param ... additional settings for \code{opts_collator}
 #'
 #' @return For \code{stri_order}, an integer vector that gives the sort order
 #' is returned.
@@ -413,20 +421,22 @@ stri_cmp_ge <- function(e1, e2, opts_collator=NULL) {
 #' @rdname stri_order
 #'
 #' @examples
-#' \donttest{
-#' stri_sort(c("hladny", "chladny"), opts_collator=stri_opts_collator(locale="pl_PL"))
+#' stri_sort(c("hladny", "chladny"), locale="pl_PL")
 #'
-#' stri_sort(c("hladny", "chladny"), opts_collator=stri_opts_collator(locale="sk_SK"))
-#' }
-stri_order <- function(str, decreasing=FALSE, na_last=TRUE, opts_collator=NULL) {
-   .Call("stri_order_or_sort", str, decreasing, na_last, opts_collator, 1L, PACKAGE="stringi")
+#' stri_sort(c("hladny", "chladny"), locale="sk_SK")
+stri_order <- function(str, decreasing=FALSE, na_last=TRUE, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_order_or_sort, str, decreasing, na_last, opts_collator, 1L)
 }
 
 
 #' @export
 #' @rdname stri_order
-stri_sort <-  function(str, decreasing=FALSE, na_last=NA, opts_collator=NULL) {
-   .Call("stri_order_or_sort", str, decreasing, na_last, opts_collator, 2L, PACKAGE="stringi")
+stri_sort <-  function(str, decreasing=FALSE, na_last=NA, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_order_or_sort, str, decreasing, na_last, opts_collator, 2L)
 }
 
 
@@ -451,23 +461,23 @@ stri_sort <-  function(str, decreasing=FALSE, na_last=NA, opts_collator=NULL) {
 #' @param opts_collator a named list with \pkg{ICU} Collator's options
 #' as generated with \code{\link{stri_opts_collator}}, \code{NULL}
 #' for default collation options
+#' @param ... additional settings for \code{opts_collator}
 #'
 #' @return Returns a character vector.
 #'
 #' @examples
-#' \donttest{
-#' # normalized and non-unicode-normalized version of the same code point:
+#' # normalized and non-Unicode-normalized version of the same code point:
 #' stri_unique(c("\u0105", stri_trans_nfkd("\u0105")))
 #' unique(c("\u0105", stri_trans_nfkd("\u0105")))
 #'
-#' stri_unique(c("gro\\u00df", "GROSS", "Gro\\u00df", "Gross"),
-#'    stri_opts_collator(strength=1))
-#' }
+#' stri_unique(c("gro\u00df", "GROSS", "Gro\u00df", "Gross"), strength=1)
 #'
 #' @family locale_sensitive
 #' @export
-stri_unique <-  function(str, opts_collator=NULL) {
-   .Call("stri_unique", str, opts_collator, PACKAGE="stringi")
+stri_unique <-  function(str, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_unique, str, opts_collator)
 }
 
 
@@ -487,7 +497,7 @@ stri_unique <-  function(str, opts_collator=NULL) {
 #' Unlike \code{\link{duplicated}} and \code{\link{anyDuplicated}},
 #' these functions test for canonical equivalence of strings
 #' (and not whether the strings are just bytewise equal)
-#' Such operations is locale-dependent.
+#' Such operations are locale-dependent.
 #' Hence, \code{stri_duplicated} and \code{stri_duplicated_any}
 #' are significantly slower (but much better suited for natural language
 #' processing) than their base R counterpart.
@@ -501,18 +511,18 @@ stri_unique <-  function(str, opts_collator=NULL) {
 #' @param opts_collator a named list with \pkg{ICU} Collator's options
 #' as generated with \code{\link{stri_opts_collator}}, \code{NULL}
 #' for default collation options
+#' @param ... additional settings for \code{opts_collator}
 #'
 #' @return
 #' \code{stri_duplicated()} returns a logical vector of the same length
-#' as \code{str}. Each of its elements indicates if an equivalent string
-#' already appeared in \code{str}.
+#' as \code{str}. Each of its elements indicates whether a canonically
+#' equivalent string was already found in \code{str}.
 #'
 #' \code{stri_duplicated_any()} returns a single non-negative integer.
 #' Value of 0 indicates that all the elements in \code{str} are unique.
 #' Otherwise, it gives the index of the first non-unique element.
 #'
 #' @examples
-#' \donttest{
 #' # In the following examples, we have 3 duplicated values,
 #' # "a" - 2 times, NA - 1 time
 #' stri_duplicated(c("a", "b", "a", NA, "a", NA))
@@ -523,22 +533,24 @@ stri_unique <-  function(str, opts_collator=NULL) {
 #' stri_duplicated(c("\u0105", stri_trans_nfkd("\u0105")))
 #' duplicated(c("\u0105", stri_trans_nfkd("\u0105")))
 #'
-#' stri_duplicated(c("gro\\u00df", "GROSS", "Gro\\u00df", "Gross"),
-#'    opts_collator=stri_opts_collator(strength=1))
-#' duplicated(c("gro\\u00df", "GROSS", "Gro\\u00df", "Gross"))
-#' }
+#' stri_duplicated(c("gro\u00df", "GROSS", "Gro\u00df", "Gross"), strength=1)
+#' duplicated(c("gro\u00df", "GROSS", "Gro\u00df", "Gross"))
 #'
 #' @rdname stri_duplicated
 #' @family locale_sensitive
 #' @export
-stri_duplicated <-  function(str, fromLast=FALSE, opts_collator=NULL) {
-   .Call("stri_duplicated", str, fromLast, opts_collator, PACKAGE="stringi")
+stri_duplicated <-  function(str, fromLast=FALSE, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_duplicated, str, fromLast, opts_collator)
 }
 
 
 #' @rdname stri_duplicated
 #' @family locale_sensitive
 #' @export
-stri_duplicated_any <-  function(str, fromLast=FALSE, opts_collator=NULL) {
-   .Call("stri_duplicated_any", str, fromLast, opts_collator, PACKAGE="stringi")
+stri_duplicated_any <-  function(str, fromLast=FALSE, ..., opts_collator=NULL) {
+   if (!missing(...))
+       opts_collator <- do.call(stri_opts_collator, as.list(c(opts_collator, ...)))
+   .Call(C_stri_duplicated_any, str, fromLast, opts_collator)
 }
