@@ -58,11 +58,13 @@
 #'
 #' @param silent suppress diagnostic messages
 #' @param check enable \code{stri_install_check()} tests
-#' @param path path to install icudt to. If \code{NULL}, then
-#' \code{file.path(find.package('stringi'), 'libs')} will be used.
+#' @param outpath path to install icudt to. If \code{NULL}, then
+#' \code{file.path(path.package("stringi"), "libs")} will be used.
+#' @param inpath path to search icudt archive in (must end with a slash).
+#' If \code{NULL}, then only stringi mirror servers will be used as possible sources
 #' Custom, non-default paths should not be used normally by \pkg{stringi} users.
 #'
-#' @return The functions return a logical value, invisibly.
+#' @return These functions return a logical value, invisibly.
 #' \code{TRUE} denotes that the requested operation has been completed
 #' successfully.
 #'
@@ -71,25 +73,19 @@
 #' \url{http://userguide.icu-project.org/icudata}
 #'
 #' @examples
-#' \donttest{stri_install_check()}
+#' stri_install_check()
 #'
-#' @rdname
-#' stri_install
+#' @rdname stri_install
 #' @export
 stri_install_check <- function(silent=FALSE) {
    stopifnot(is.logical(silent), length(silent) == 1)
 
    allok <- tryCatch({
-      if (!silent)
-         message(stri_info(TRUE)) # this may also throw an error
+      if (!silent) message(stri_info(TRUE)) # this may also throw an error
 
-      if (length(stri_enc_list()) <= 0) stop("encodings unsupported")
+      if (length(stri_enc_list()) <= 0)    stop("encodings unsupported")
       if (length(stri_locale_list()) <= 0) stop("locales unsupported")
-      if (length(stri_trans_list()) <= 0) stop("transliterators unsupported")
-      if (stri_cmp("a", "b", opts_collator=stri_opts_collator(locale="en_US")) != -1)
-         stop("no collator rules installed")
-      if (stri_detect_regex("123abc!@#", "\\p{L}") != TRUE)
-         stop("regex engine failure")
+      if (length(stri_trans_list()) <= 0)  stop("transliterators unsupported")
       TRUE
    }, error=function(e) { FALSE })
 
@@ -105,61 +101,83 @@ stri_install_check <- function(silent=FALSE) {
 }
 
 
-#' @rdname
-#' stri_install
+#' @rdname stri_install
 #' @export
-stri_install_icudt <- function(check=TRUE, path=NULL) {
-   stopifnot(is.logical(check), length(check) == 1)
+#' @importFrom tools md5sum
+stri_install_icudt <- function(check=TRUE, outpath=NULL, inpath=NULL) {
+   stopifnot(is.logical(check), length(check) == 1, !is.na(check))
    if (check && stri_install_check(TRUE)) {
       message("icudt has been already installed.")
       return(invisible(TRUE))
    }
 
-   if (is.null(path))
-      path <- file.path(find.package('stringi'), 'libs')
-   stopifnot(is.character(path), length(path) == 1)
+   if (is.null(outpath))
+      outpath <- file.path(path.package("stringi"), "libs")
+   stopifnot(is.character(outpath), length(outpath) == 1, file.exists(outpath))
 
-   mirror1 <- "http://static.rexamine.com/packages/"
-   mirror2 <- "http://www.mini.pw.edu.pl/~gagolews/stringi/"
-   mirror3 <- "http://www.ibspan.waw.pl/~gagolews/stringi/"
+   fname <- if (.Platform$endian == 'little') "icudt52l.zip"
+                                         else "icudt52b.zip"
 
+   md5ex <- if (.Platform$endian == 'little') "d86d3191818ae58d5703f1ac78b0050c"
+                                         else "91cd9aacaa4e776bd14f20c8839cd9c2"
 
-   if (.Platform$endian == 'little') {
-      fname <- "icudt52l.zip"
-   }
-   else {
-      fname <- "icudt52b.zip"
+   mirrors <- c("http://static.rexamine.com/packages/",
+                "http://www.mini.pw.edu.pl/~gagolews/stringi/",
+                "http://www.ibspan.waw.pl/~gagolews/stringi/")
+
+   if (!is.null(inpath)) {
+      stopifnot(is.character(inpath), length(inpath) > 0, !is.na(inpath))
+      mirrors <- c(inpath, mirrors)
    }
 
    outfname <- tempfile(fileext=".zip")
-   download_from_mirror <- function(mirror, outfname) {
+   download_from_mirror <- function(href, outfname) {
       tryCatch({
-         ret <- download.file(paste0(mirror, fname), outfname, mode="wb")
-         if (ret != 0) stop("download error")
-         if (!file.exists(outfname)) stop("download error")
+         suppressWarnings(file.remove(outfname))
+         if (!grepl("^https?://", href)) {
+            # try to copy icudt from a local repo
+            if (!file.exists(href)) return("no icudt in a local repo")
+            message("icudt has been found in a local repo")
+            file.copy(href, outfname)
+         }
+         else {
+            # download icudt
+            if (download.file(href, outfname, mode="wb") != 0)
+               return("download error")
+         }
+         if (!file.exists(outfname)) return("download error")
+         md5ob <- tools::md5sum(outfname)
+         if (is.na(md5ob)) return("error checking md5sum")
+         if (md5ob != md5ex) return("md5sum mismatch")
          TRUE
-      }, error = function(e) FALSE)
+      }, error = function(e) as.character(e))
    }
 
    message("downloading ICU data library (icudt)")
-   message("the files will be extracted to: ", path)
-   allok <- download_from_mirror(mirror1, outfname)
-   allok <- allok || download_from_mirror(mirror2, outfname)
-   allok <- allok || download_from_mirror(mirror3, outfname)
+   message("the files will be extracted to: ", outpath)
+   allok <- FALSE
+   for (m in mirrors) {
+      if (identical(status <- download_from_mirror(paste0(m, fname), outfname), TRUE)) {
+         allok <- TRUE
+         break
+      }
+      else message(status)
+   }
 
    if (!allok) {
-      message("download failed")
+      message("icudt download failed")
       return(invisible(FALSE))
    }
-   message("download OK")
+   message("icudt fetch OK")
 
    message("decompressing downloaded archive")
-   res <- unzip(outfname, exdir=path)
+   res <- unzip(outfname, exdir=outpath, overwrite=TRUE)
    if (!is.character(res) || length(res) <= 0) {
       message("error decompressing archive")
       return(invisible(FALSE))
    }
 
+   suppressWarnings(file.remove(outfname))
    message("icudt has been installed successfully")
    message("restart R to apply changes")
    invisible(TRUE)
