@@ -59,10 +59,10 @@
 #' @return Returns an integer vector of the same length as \code{str}.
 #'
 #' @examples
-#' \donttest{
 #' stri_numbytes(letters)
 #' stri_numbytes(c('abc', '123', '\u0105\u0104'))
 #'
+#' \dontrun{
 #' # this used to fail on Windows, as there was no native support for 4-bytes
 #' # Unicode characters; see, however, stri_escape_unicode():
 #' stri_numbytes('\U7fffffff') # compare stri_length('\U7fffffff')
@@ -71,7 +71,7 @@
 #' @export
 #' @family length
 stri_numbytes <- function(str) {
-   .Call("stri_numbytes", str, PACKAGE="stringi")
+   .Call(C_stri_numbytes, str)
 }
 
 
@@ -103,21 +103,18 @@ stri_numbytes <- function(str) {
 #' @return Returns an integer vector of the same length as \code{str}.
 #'
 #' @examples
-#' \donttest{
 #' stri_length(LETTERS)
 #' stri_length(c('abc', '123', '\u0105\u0104'))
 #' stri_length('\u0105') # length is one, but...
 #' stri_numbytes('\u0105') # 2 bytes are used
 #' stri_numbytes(stri_trans_nfkd('\u0105')) # 3 bytes here but...
 #' stri_length(stri_trans_nfkd('\u0105')) # ...two code points (!)
-#' stri_count_boundaries(stri_trans_nfkd('\u0105'),
-#'    stri_opts_brkiter(type="character")) # ...and one Unicode character
-#' }
+#' stri_count_boundaries(stri_trans_nfkd('\u0105'), type="character") # ...and one Unicode character
 #'
 #' @export
 #' @family length
 stri_length <- function(str) {
-   .Call("stri_length", str, PACKAGE="stringi")
+   .Call(C_stri_length, str)
 }
 
 
@@ -136,14 +133,12 @@ stri_length <- function(str) {
 #' @return Returns a logical vector of the same length as \code{str}.
 #'
 #' @examples
-#' \donttest{
 #' stri_isempty(letters[1:3])
 #' stri_isempty(c(',', '', 'abc', '123', '\u0105\u0104'))
 #' stri_isempty(character(1))
-#' }
 #'
 #' @export
 #' @family length
 stri_isempty <- function(str) {
-   .Call("stri_isempty", str, PACKAGE="stringi")
+   .Call(C_stri_isempty, str)
 }
