@@ -47,13 +47,11 @@
 #' @return Returns a character vector.
 #'
 #' @examples
-#' \donttest{
 #' stri_reverse(c("123", "abc d e f"))
 #' stri_reverse("ZXY (\u0105\u0104123$^).")
 #' stri_reverse(stri_trans_nfd('\u0105')) == stri_trans_nfd('\u0105') # A, ogonek -> agonek, A
-#' }
 #'
 #' @export
 stri_reverse <- function(str) {
-   .Call("stri_reverse", str, PACKAGE="stringi")
+   .Call(C_stri_reverse, str)
 }
