@@ -84,16 +84,14 @@
 #' @export
 #'
 #' @examples
-#' \donttest{
 #' stri_trans_general("gro\u00df", "latin-ascii")
 #' stri_trans_general("stringi", "latin-greek")
 #' stri_trans_general("stringi", "latin-cyrillic")
 #' stri_trans_general("stringi", "upper") # see stri_trans_toupper
 #' stri_trans_general("\u0104", "nfd; lower") # compound id; see stri_trans_nfd
 #' stri_trans_general("tato nie wraca ranki wieczory", "pl-pl_FONIPA")
-#' }
 stri_trans_general <- function(str, id) {
-   .Call("stri_trans_general", str, id, PACKAGE="stringi")
+   .Call(C_stri_trans_general, str, id)
 }
 
 
@@ -114,5 +112,5 @@ stri_trans_general <- function(str, id) {
 #' @family transform
 #' @export
 stri_trans_list <- function() {
-   .Call("stri_trans_list", PACKAGE="stringi")
+   .Call(C_stri_trans_list)
 }
