@@ -61,11 +61,11 @@ SEXP stri_info()
 
    STRI__PROTECT(vals = Rf_allocVector(VECSXP, infosize));
    SET_VECTOR_ELT(vals, 0, Rf_mkString(U_UNICODE_VERSION));
-   SET_VECTOR_ELT(vals, 1, Rf_mkString(U_ICU_VERSION ));
-   SET_VECTOR_ELT(vals, 2, stri_locale_info(R_NilValue));
+   SET_VECTOR_ELT(vals, 1, Rf_mkString(U_ICU_VERSION));
+   SET_VECTOR_ELT(vals, 2, stri_locale_info(R_NilValue)); // may call Rf_error
    SET_VECTOR_ELT(vals, 3,
       stri__make_character_vector(2, "UTF-8", "UTF-16")); // fixed strings
-   SET_VECTOR_ELT(vals, 4, stri_enc_info(R_NilValue));
+   SET_VECTOR_ELT(vals, 4, stri_enc_info(R_NilValue));  // may call Rf_error
 
    stri__set_names(vals, infosize,
       "Unicode.version", "ICU.version", "Locale",
