@@ -44,17 +44,21 @@ StriContainerByteSearch::StriContainerByteSearch()
 {
    this->patternLen = 0;
    this->patternStr = NULL;
+   this->patternLenCaseInsensitive = 0;
+   this->patternStrCaseInsensitive = NULL;
    this->searchPos = -1;
+   this->searchEnd = -1;
    this->searchStr = NULL;
    this->searchLen = 0;
+   this->flags = 0;
 #ifndef NDEBUG
    this->debugMatcherIndex = -1;
 #endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
+//#ifndef STRI__BYTESEARCH_DISABLE_KMP
    this->kmpMaxSize = 0;
    this->patternPos = -1;
    this->kmpNext = NULL;
-#endif
+//#endif
 }
 
 
@@ -63,22 +67,37 @@ StriContainerByteSearch::StriContainerByteSearch()
  * @param rstr R character vector
  * @param _nrecycle extend length [vectorization]
  */
-StriContainerByteSearch::StriContainerByteSearch(SEXP rstr, R_len_t _nrecycle)
+StriContainerByteSearch::StriContainerByteSearch(SEXP rstr, R_len_t _nrecycle, uint32_t _flags)
    : StriContainerUTF8(rstr, _nrecycle, true)
 {
+   this->flags = _flags;
+
    this->patternLen = 0;
    this->patternStr = NULL;
+
    this->searchPos = -1;
+   this->searchEnd = -1;
    this->searchStr = NULL;
    this->searchLen = 0;
+
 #ifndef NDEBUG
    this->debugMatcherIndex = -1;
 #endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
+
+//#ifndef STRI__BYTESEARCH_DISABLE_KMP
    this->patternPos = -1;
    this->kmpMaxSize = getMaxNumBytes()+1;
    this->kmpNext = new int[kmpMaxSize];
-#endif
+   if (!this->kmpNext) throw StriException(MSG__MEM_ALLOC_ERROR);
+//#endif
+
+   this->patternLenCaseInsensitive = 0;
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      this->patternStrCaseInsensitive = new UChar32[this->kmpMaxSize];
+      if (!this->patternStrCaseInsensitive) throw StriException(MSG__MEM_ALLOC_ERROR);
+   }
+   else
+      this->patternStrCaseInsensitive = NULL;
 }
 
 
@@ -91,16 +110,27 @@ StriContainerByteSearch::StriContainerByteSearch(StriContainerByteSearch& contai
    this->patternLen = 0;
    this->patternStr = NULL;
    this->searchPos = -1;
+   this->searchEnd = -1;
    this->searchStr = NULL;
    this->searchLen = 0;
+   this->flags = container.flags;
 #ifndef NDEBUG
    this->debugMatcherIndex = -1;
 #endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
+//#ifndef STRI__BYTESEARCH_DISABLE_KMP
    this->patternPos = -1;
    this->kmpMaxSize = container.kmpMaxSize;
    this->kmpNext = new int[kmpMaxSize];
-#endif
+   if (!this->kmpNext) throw StriException(MSG__MEM_ALLOC_ERROR);
+
+   this->patternLenCaseInsensitive = 0;
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      this->patternStrCaseInsensitive = new UChar32[this->kmpMaxSize];
+      if (!this->patternStrCaseInsensitive) throw StriException(MSG__MEM_ALLOC_ERROR);
+   }
+   else
+      this->patternStrCaseInsensitive = NULL;
+//#endif
 }
 
 
@@ -115,16 +145,27 @@ StriContainerByteSearch& StriContainerByteSearch::operator=(StriContainerByteSea
    this->patternLen = 0;
    this->patternStr = NULL;
    this->searchPos = -1;
+   this->searchEnd = -1;
    this->searchStr = NULL;
    this->searchLen = 0;
+   this->flags = container.flags;
 #ifndef NDEBUG
    this->debugMatcherIndex = -1;
 #endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
+//#ifndef STRI__BYTESEARCH_DISABLE_KMP
    this->patternPos = -1;
    this->kmpMaxSize = container.kmpMaxSize;
    this->kmpNext = new int[kmpMaxSize];
-#endif
+   if (!this->kmpNext) throw StriException(MSG__MEM_ALLOC_ERROR);
+
+   this->patternLenCaseInsensitive = 0;
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      this->patternStrCaseInsensitive = new UChar32[this->kmpMaxSize];
+      if (!this->patternStrCaseInsensitive) throw StriException(MSG__MEM_ALLOC_ERROR);
+   }
+   else
+      this->patternStrCaseInsensitive = NULL;
+//#endif
    return *this;
 }
 
@@ -135,30 +176,68 @@ StriContainerByteSearch& StriContainerByteSearch::operator=(StriContainerByteSea
 StriContainerByteSearch::~StriContainerByteSearch()
 {
    // nothing to clean
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-   if (kmpNext)
+//#ifndef STRI__BYTESEARCH_DISABLE_KMP
+   if (kmpNext) {
       delete [] kmpNext;
-#endif
+      kmpNext = NULL;
+   }
+
+   if (patternStrCaseInsensitive) {
+      delete [] patternStrCaseInsensitive;
+      patternStrCaseInsensitive = NULL;
+   }
+//#endif
+}
+
+
+/** Create KMP table for rev iteration [case insensitive search]
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+*/
+void StriContainerByteSearch::createKMPtableBackCaseInsensitive()
+{
+   kmpNext[0] = -1;
+   for (R_len_t i=0; i<patternLenCaseInsensitive; ++i) {
+      kmpNext[i+1] = kmpNext[i]+1;
+      while (kmpNext[i+1] > 0 &&
+            patternStrCaseInsensitive[patternLen-i-1] != patternStrCaseInsensitive[patternLenCaseInsensitive-(kmpNext[i+1]-1)-1])
+         kmpNext[i+1] = kmpNext[kmpNext[i+1]-1]+1;
+   }
 }
 
 
 /** Create KMP table for rev iteration
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *          KMP upgrade
 */
 void StriContainerByteSearch::createKMPtableBack()
 {
-   int k = 0, j = -1;
    kmpNext[0] = -1;
-   while (k < patternLen) {
-      while (j > -1 && patternStr[patternLen-1-k] != patternStr[patternLen-1-j])
-         j = kmpNext[j];
-      k++;
-      j++;
-      if (patternStr[patternLen-1-k] == patternStr[patternLen-1-j])
-         kmpNext[k] = kmpNext[j];
-      else
-         kmpNext[k] = j;
+   for (R_len_t i=0; i<patternLen; ++i) {
+      kmpNext[i+1] = kmpNext[i]+1;
+      while (kmpNext[i+1] > 0 &&
+            patternStr[patternLen-i-1] != patternStr[patternLen-(kmpNext[i+1]-1)-1])
+         kmpNext[i+1] = kmpNext[kmpNext[i+1]-1]+1;
+   }
+}
+
+
+/** Create KMP table for fwd iteration [case insensitive]
+ *
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+*/
+void StriContainerByteSearch::createKMPtableFwdCaseInsensitive()
+{
+   kmpNext[0] = -1;
+   for (R_len_t i=0; i<patternLenCaseInsensitive; ++i) {
+      kmpNext[i+1] = kmpNext[i]+1;
+      while (kmpNext[i+1] > 0 &&
+            patternStrCaseInsensitive[i] != patternStrCaseInsensitive[kmpNext[i+1]-1])
+         kmpNext[i+1] = kmpNext[kmpNext[i+1]-1]+1;
    }
 }
 
@@ -169,20 +248,18 @@ void StriContainerByteSearch::createKMPtableBack()
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
  *          KMP upgrade; special procedure for patternLen <= 4
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *          KMP upgrade
 */
 void StriContainerByteSearch::createKMPtableFwd()
 {
-   int k = 0, j = -1;
    kmpNext[0] = -1;
-   while (k < patternLen) {
-      while (j > -1 && patternStr[k] != patternStr[j])
-         j = kmpNext[j];
-      k++;
-      j++;
-      if (patternStr[k] == patternStr[j])
-         kmpNext[k] = kmpNext[j];
-      else
-         kmpNext[k] = j;
+   for (R_len_t i=0; i<patternLen; ++i) {
+      kmpNext[i+1] = kmpNext[i]+1;
+      while (kmpNext[i+1] > 0 &&
+            patternStr[i] != patternStr[kmpNext[i+1]-1])
+         kmpNext[i+1] = kmpNext[kmpNext[i+1]-1]+1;
    }
 }
 
@@ -209,15 +286,16 @@ void StriContainerByteSearch::setupMatcherBack(R_len_t i, const char* _searchStr
       this->patternStr = get(i).c_str();
       this->patternLen = get(i).length();
 
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
-      if (this->patternLen > 4) { // a short pattern => don't use KMP
-#endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-         createKMPtableBack();
-#endif
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
+      if (flags & BYTESEARCH_CASE_INSENSITIVE) {
+         upgradePatternCaseInsensitive();
+         createKMPtableBackCaseInsensitive();
       }
+      else {
+#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
+         if (this->patternLen > 4) // a short pattern => don't use KMP
+            createKMPtableBack();
 #endif
+      }
    }
 
    this->searchStr = _searchStr;
@@ -259,15 +337,16 @@ void StriContainerByteSearch::setupMatcherFwd(R_len_t i, const char* _searchStr,
       this->patternStr = get(i).c_str();
       this->patternLen = get(i).length();
 
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
-      if (this->patternLen > 4) { // a short pattern => don't use KMP
-#endif
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-         createKMPtableFwd();
-#endif
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
+      if (flags & BYTESEARCH_CASE_INSENSITIVE) {
+         upgradePatternCaseInsensitive();
+         createKMPtableFwdCaseInsensitive();
       }
+      else {
+#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
+         if (this->patternLen > 4) // a short pattern => don't use KMP
+            createKMPtableFwd();
 #endif
+      }
    }
 
    this->searchStr = _searchStr;
@@ -292,9 +371,8 @@ void StriContainerByteSearch::resetMatcher()
 #endif
 
    this->searchPos = -1;
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
+   this->searchEnd = -1;
    this->patternPos = -1;
-#endif
 }
 
 
@@ -308,13 +386,23 @@ void StriContainerByteSearch::resetMatcher()
  *
  * @version 0.2-4 (Marek Gagolewski, 2014-05-15)
  *          BUGFIX: load of misaligned adresses
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-11-30)
+ *          BUGFIX: ret USEARCH_DONE immediately if startPos is too far away
  */
 R_len_t StriContainerByteSearch::findFromPosFwd_short(R_len_t startPos)
 {
+   if (startPos > searchLen-patternLen) { // this check is OK, we do a case-sensitive search
+      searchPos = searchEnd = searchLen;
+      return USEARCH_DONE;
+   }
+
    if (patternLen == 1) {
+         // else not found
       unsigned char pat = (unsigned char)patternStr[0];
       for (searchPos = startPos; searchPos<searchLen-1+1; ++searchPos) {
          if (pat == (unsigned char)searchStr[searchPos]) {
+            searchEnd = searchPos + 1;
             return searchPos;
          }
       }
@@ -341,6 +429,7 @@ R_len_t StriContainerByteSearch::findFromPosFwd_short(R_len_t startPos)
          cur |= (uint16_t)(*curstr);
          ++curstr;
          if (pat == cur) {
+            searchEnd = searchPos + 2;
             return searchPos;
          }
       }
@@ -377,6 +466,7 @@ R_len_t StriContainerByteSearch::findFromPosFwd_short(R_len_t startPos)
          cur |= (uint32_t)(*curstr);
          ++curstr;
          if ((pat&mask) == (cur&mask)) {
+            searchEnd = searchPos + 3;
             return searchPos;
          }
       }
@@ -415,42 +505,43 @@ R_len_t StriContainerByteSearch::findFromPosFwd_short(R_len_t startPos)
          cur |= (uint32_t)(*curstr);
          ++curstr;
          if (pat == cur) {
+            searchEnd = searchPos + 4;
             return searchPos;
          }
       }
    }
    // else not found
-   searchPos = searchLen;
+   searchPos = searchEnd = searchLen;
    return USEARCH_DONE;
 }
 
 
-/** find first match - naive search
- *
- * @param startPos where to start
- * @return USEARCH_DONE on no match, otherwise start index
- *
- * @version 0.1-?? (Marek Gagolewski)
- *
- * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
- *          separate method
- */
-R_len_t StriContainerByteSearch::findFromPosFwd_naive(R_len_t startPos)
-{
-   // Naive search algorithm
-   for (searchPos = startPos; searchPos<searchLen-patternLen+1; ++searchPos) {
-      R_len_t k=0;
-      while (k<patternLen && searchStr[searchPos+k] == patternStr[k])
-         k++;
-      if (k == patternLen) {
-         // found!
-         return searchPos;
-      }
-   }
-   // not found
-   searchPos = searchLen;
-   return USEARCH_DONE;
-}
+///** find first match - naive search
+// *
+// * @param startPos where to start
+// * @return USEARCH_DONE on no match, otherwise start index
+// *
+// * @version 0.1-?? (Marek Gagolewski)
+// *
+// * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
+// *          separate method
+// */
+//R_len_t StriContainerByteSearch::findFromPosFwd_naive(R_len_t startPos)
+//{
+//   // Naive search algorithm
+//   for (searchPos = startPos; searchPos<searchLen-patternLen+1; ++searchPos) {
+//      R_len_t k=0;
+//      while (k<patternLen && searchStr[searchPos+k] == patternStr[k])
+//         k++;
+//      if (k == patternLen) {
+//         // found!
+//         return searchPos;
+//      }
+//   }
+//   // not found
+//   searchPos = searchLen;
+//   return USEARCH_DONE;
+//}
 
 
 /** find first match - KMP
@@ -463,169 +554,55 @@ R_len_t StriContainerByteSearch::findFromPosFwd_naive(R_len_t startPos)
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
  *          KMP upgraded; separate method
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    use BYTESEARCH_CASE_INSENSITIVE
  */
 R_len_t StriContainerByteSearch::findFromPosFwd_KMP(R_len_t startPos)
 {
    int j = startPos;
    patternPos = 0;
-   while (j < searchLen) {
-      while (patternPos >= 0 && patternStr[patternPos] != searchStr[j])
-         patternPos = kmpNext[patternPos];
-      patternPos++;
-      j++;
-      if (patternPos >= patternLen) {
-         searchPos = j-patternPos;
-         return searchPos;
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      UChar32 c = 0;
+      while (j < searchLen) {
+         U8_NEXT(searchStr, j, searchLen, c);
+         c = u_toupper(c);
+         while (patternPos >= 0 && patternStrCaseInsensitive[patternPos] != c)
+            patternPos = kmpNext[patternPos];
+         patternPos++;
+         if (patternPos == patternLenCaseInsensitive) {
+            searchEnd = j;
+
+            // we need to go back by patternLenCaseInsensitive code points
+            R_len_t k = patternLenCaseInsensitive;
+            searchPos = j;
+            while (k > 0) {
+               U8_BACK_1((const uint8_t*)searchStr, 0, searchPos);
+               k--;
+            }
+            return searchPos;
+         }
+      }
+   }
+   else {
+      while (j < searchLen) {
+         while (patternPos >= 0 && patternStr[patternPos] != searchStr[j])
+            patternPos = kmpNext[patternPos];
+         patternPos++;
+         j++;
+         if (patternPos == patternLen) {
+            searchEnd = j;
+            searchPos = j-patternLen;
+            return searchPos;
+         }
       }
    }
    // else not found
-   searchPos = searchLen;
+   searchPos = searchEnd = searchLen;
    return USEARCH_DONE;
 }
 
 
-/** find first match
- *
- * resets the matcher
- *
- * @return USEARCH_DONE on no match, otherwise start index
- *
- * @version 0.1-?? (Marek Gagolewski)
- *
- * @version 0.1-?? (Bartek Tartanus, 2013-08-15)
- *          uses KMP
- *
- * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
- *          KMP upgraded and now used by default;
- *          special procedure for patternLen <= 4
- */
-R_len_t StriContainerByteSearch::findFirst()
-{
-   // "Any byte oriented string searching algorithm can be used with
-   // UTF-8 data, since the sequence of bytes for a character cannot
-   // occur anywhere else."
-#ifndef NDEBUG
-   if (!this->searchStr || !this->patternStr)
-      throw StriException("DEBUG: StriContainerByteSearch: setupMatcher() hasn't been called yet");
-#endif
-
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
-   if (patternLen <= 4)
-      return findFromPosFwd_short(0);
-#endif
-
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-   return findFromPosFwd_KMP(0);
-#else
-   return findFromPosFwd_naive(0);
-#endif
-}
-
-
-/** find next match
- *
- * continues previous search
- *
- * @return USEARCH_DONE on no match, otherwise start index
- *
- * @version 0.1-?? (Marek Gagolewski)
- *
- * @version 0.1-?? (Bartek Tartanus, 2013-08-15)
- *          uses KMP
- *
- * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
- *          KMP upgraded and now used by default;
- *          use findFromPosFwd
- */
-R_len_t StriContainerByteSearch::findNext()
-{
-#ifndef NDEBUG
-   if (!this->searchStr || !this->patternStr)
-      throw StriException("DEBUG: StriContainerByteSearch: setupMatcher() hasn't been called yet");
-#endif
-
-   if (searchPos < 0) return findFirst();
-
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
-   if (patternLen <= 4)
-      return findFromPosFwd_short(searchPos+patternLen);
-#endif
-
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-   return findFromPosFwd_KMP(searchPos+patternLen);
-#else
-   return findFromPosFwd_naive(searchPos+patternLen);
-#endif
-}
-
-
-/** find last match
- *
- * resets the matcher
- *
- * @return USEARCH_DONE on no match, otherwise start index
- *
- * @version 0.1-?? (Marek Gagolewski)
- *
- * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
- *          Using KNP
- */
-R_len_t StriContainerByteSearch::findLast()
-{
-#ifndef NDEBUG
-   if (!this->searchStr || !this->patternStr)
-      throw StriException("DEBUG: StriContainerByteSearch: setupMatcher() hasn't been called yet");
-#endif
-
-#ifndef STRI__BYTESEARCH_DISABLE_SHORTPAT
-   if (patternLen <= 4)
-      return findFromPosBack_short(searchLen-1);
-#endif
-
-#ifndef STRI__BYTESEARCH_DISABLE_KMP
-   return findFromPosBack_KMP(searchLen-1);
-#else
-   return findFromPosBack_naive(searchLen-1);
-#endif
-}
-
-
-/** get start index of pattern match from the last search
- *
- * @return byte index in searchStr
- */
-R_len_t StriContainerByteSearch::getMatchedStart()
-{
-#ifndef NDEBUG
-   if (!this->searchStr || !this->patternStr)
-      throw StriException("DEBUG: StriContainerByteSearch: setupMatcher() hasn't been called yet");
-#endif
-
-   if (searchPos < 0 || searchPos > searchLen-patternLen)
-      throw StriException("StriContainerByteSearch: no match at current position! This is a BUG.");
-
-   return searchPos;
-}
-
-
-/** get length of pattern match from the last search
- *
- * @return byte index in searchStr
- */
-R_len_t StriContainerByteSearch::getMatchedLength()
-{
-#ifndef NDEBUG
-   if (!this->searchStr || !this->patternStr)
-      throw StriException("DEBUG: StriContainerByteSearch: setupMatcher() hasn't been called yet");
-#endif
-
-   if (searchPos < 0 || searchPos > searchLen-patternLen)
-      throw StriException("StriContainerByteSearch: no match at current position! This is a BUG.");
-
-   return patternLen; // trivial :>
-}
-
-
 /** find last match - case of short pattern
  *
  * @param startPos where to start
@@ -635,13 +612,23 @@ R_len_t StriContainerByteSearch::getMatchedLength()
  *
  * @version 0.2-4 (Marek Gagolewski, 2014-05-15)
  *          BUGFIX: load of misaligned adresses
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-11-30)
+ *          BUGFIX: ret USEARCH_DONE immediately if startPos indicates no match
+ *
  */
 R_len_t StriContainerByteSearch::findFromPosBack_short(R_len_t startPos)
 {
+   if (startPos+1 < patternLen) { // check OK, case-sensitive search
+      searchPos = searchEnd = searchLen;
+      return USEARCH_DONE;
+   }
+
    if (patternLen == 1) {
       unsigned char pat = (unsigned char)patternStr[0];
       for (searchPos = startPos-0; searchPos>=0; --searchPos) {
          if (pat == (unsigned char)searchStr[searchPos]) {
+            searchEnd = searchPos + 1;
             return searchPos;
          }
       }
@@ -660,6 +647,7 @@ R_len_t StriContainerByteSearch::findFromPosBack_short(R_len_t startPos)
          cur |= (uint16_t)(*curstr);
          --curstr;
          if (pat == cur) {
+            searchEnd = searchPos + 2;
             return searchPos;
          }
       }
@@ -685,6 +673,7 @@ R_len_t StriContainerByteSearch::findFromPosBack_short(R_len_t startPos)
          cur |= (uint32_t)(*curstr);
          --curstr;
          if ((pat&mask) == (cur&mask)) {
+            searchEnd = searchPos + 3;
             return searchPos;
          }
       }
@@ -714,39 +703,40 @@ R_len_t StriContainerByteSearch::findFromPosBack_short(R_len_t startPos)
          cur |= (uint32_t)(*curstr);
          --curstr;
          if (pat == cur) {
+            searchEnd = searchPos + 4;
             return searchPos;
          }
       }
    }
    // else not found
-   searchPos = searchLen;
+   searchPos = searchEnd = searchLen;
    return USEARCH_DONE;
 }
 
 
-/** find last match - naive search
- *
- * @param startPos where to start
- * @return USEARCH_DONE on no match, otherwise start index
- *
- * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
- */
-R_len_t StriContainerByteSearch::findFromPosBack_naive(R_len_t startPos)
-{
-   // Naive search algorithm
-   for (searchPos = startPos-patternLen+1; searchPos>=0; --searchPos) {
-      R_len_t k=0;
-      while (k<patternLen && searchStr[searchPos+k] == patternStr[k])
-         k++;
-      if (k == patternLen) {
-         // found!
-         return searchPos;
-      }
-   }
-   // not found
-   searchPos = searchLen;
-   return USEARCH_DONE;
-}
+///** find last match - naive search
+// *
+// * @param startPos where to start
+// * @return USEARCH_DONE on no match, otherwise start index
+// *
+// * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
+// */
+//R_len_t StriContainerByteSearch::findFromPosBack_naive(R_len_t startPos)
+//{
+//   // Naive search algorithm
+//   for (searchPos = startPos-patternLen+1; searchPos>=0; --searchPos) {
+//      R_len_t k=0;
+//      while (k<patternLen && searchStr[searchPos+k] == patternStr[k])
+//         k++;
+//      if (k == patternLen) {
+//         // found!
+//         return searchPos;
+//      }
+//   }
+//   // not found
+//   searchPos = searchLen;
+//   return USEARCH_DONE;
+//}
 
 
 /** find last match - KMP
@@ -755,22 +745,173 @@ R_len_t StriContainerByteSearch::findFromPosBack_naive(R_len_t startPos)
  * @return USEARCH_DONE on no match, otherwise start index
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-11)
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    use BYTESEARCH_CASE_INSENSITIVE
  */
 R_len_t StriContainerByteSearch::findFromPosBack_KMP(R_len_t startPos)
 {
    int j = startPos;
    patternPos = 0;
-   while (j >= 0) {
-      while (patternPos >= 0 && patternStr[patternLen-1-patternPos] != searchStr[j])
-         patternPos = kmpNext[patternPos];
-      patternPos++;
-      j--;
-      if (patternPos >= patternLen) {
-         searchPos = j+1;
-         return searchPos;
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      while (j > 0) {
+         UChar32 c;
+         U8_PREV(searchStr, 0, j, c);
+         c = u_toupper(c);
+         while (patternPos >= 0 &&
+               patternStrCaseInsensitive[patternLenCaseInsensitive-1-patternPos] != c)
+            patternPos = kmpNext[patternPos];
+         patternPos++;
+         if (patternPos == patternLenCaseInsensitive) {
+            searchPos = j;
+
+            // we need to go forward by patternLenCaseInsensitive code points
+            R_len_t k = patternLenCaseInsensitive;
+            searchEnd = j;
+            while (k > 0) {
+               U8_FWD_1((const uint8_t*)searchStr, searchEnd, searchLen);
+               k--;
+            }
+
+            return searchPos;
+         }
       }
    }
+   else {
+      while (j > 0) {
+         j--;
+         while (patternPos >= 0 && patternStr[patternLen-1-patternPos] != searchStr[j])
+            patternPos = kmpNext[patternPos];
+         patternPos++;
+         if (patternPos == patternLen) {
+            searchEnd = j+patternLen;
+            searchPos = j;
+            return searchPos;
+         }
+      }
+   }
+
    // else not found
-   searchPos = searchLen;
+   searchPos = searchEnd = searchLen;
    return USEARCH_DONE;
 }
+
+
+/** Read settings flags from a list
+ *
+ * may call Rf_error
+ *
+ * @param opts_fixed list
+ * @param allow_overlap
+ * @return flags
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-08)
+ *    add `overlap` option
+ */
+uint32_t StriContainerByteSearch::getByteSearchFlags(SEXP opts_fixed, bool allow_overlap)
+{
+   uint32_t flags = 0;
+   if (!isNull(opts_fixed) && !Rf_isVectorList(opts_fixed))
+      Rf_error(MSG__ARG_EXPECTED_LIST, "opts_fixed"); // error() call allowed here
+
+   R_len_t narg = isNull(opts_fixed)?0:LENGTH(opts_fixed);
+
+   if (narg > 0) {
+
+      SEXP names = Rf_getAttrib(opts_fixed, R_NamesSymbol);
+      if (names == R_NilValue || LENGTH(names) != narg)
+         Rf_error(MSG__FIXED_CONFIG_FAILED); // error() call allowed here
+
+      for (R_len_t i=0; i<narg; ++i) {
+         if (STRING_ELT(names, i) == NA_STRING)
+            Rf_error(MSG__FIXED_CONFIG_FAILED); // error() call allowed here
+
+         const char* curname = CHAR(STRING_ELT(names, i));
+
+         if  (!strcmp(curname, "case_insensitive")) {
+            bool val = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_fixed, i), "case_insensitive");
+            if (val) flags |= BYTESEARCH_CASE_INSENSITIVE;
+         } else if  (!strcmp(curname, "overlap") && allow_overlap) {
+            bool val = stri__prepare_arg_logical_1_notNA(VECTOR_ELT(opts_fixed, i), "overlap");
+            if (val) flags |= BYTESEARCH_OVERLAP;
+         } else {
+            Rf_warning(MSG__INCORRECT_FIXED_OPTION, curname);
+         }
+      }
+   }
+
+   return flags;
+}
+
+
+/**
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ */
+void StriContainerByteSearch::upgradePatternCaseInsensitive()
+{
+   UChar32 c = 0;
+   R_len_t j = 0;
+   patternLenCaseInsensitive = 0;
+   while (j < patternLen) {
+      U8_NEXT(patternStr, j, patternLen, c);
+#ifndef NDEBUG
+      if (patternLenCaseInsensitive >= this->kmpMaxSize)
+         throw StriException("!NDEBUG: StriContainerByteSearch::upgradePatternCaseInsensitive()");
+#endif
+      patternStrCaseInsensitive[patternLenCaseInsensitive++] = u_toupper(c);
+   }
+   patternStrCaseInsensitive[patternLenCaseInsensitive] = 0;
+}
+
+
+/**
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ */
+bool StriContainerByteSearch::endsWith(R_len_t byteindex)
+{
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      for (R_len_t k = 0; k < patternLenCaseInsensitive; ++k) {
+         UChar32 c;
+         U8_PREV(searchStr, 0, byteindex, c);
+         c = u_toupper(c);
+         if (patternStrCaseInsensitive[patternLenCaseInsensitive-k-1] != c)
+            return false;
+      }
+   }
+   else {
+      for (R_len_t k=0; k < patternLen; ++k)
+         if (searchStr[byteindex-k-1] != patternStr[patternLen-k-1])
+            return false;
+   }
+
+   return true; // found
+}
+
+
+/**
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ */
+bool StriContainerByteSearch::startsWith(R_len_t byteindex)
+{
+   if (flags&BYTESEARCH_CASE_INSENSITIVE) {
+      for (R_len_t k = 0; k < patternLenCaseInsensitive; ++k) {
+         UChar32 c;
+         U8_NEXT(searchStr, byteindex, searchLen, c);
+         c = u_toupper(c);
+         if (patternStrCaseInsensitive[k] != c)
+            return false;
+      }
+   }
+   else {
+      for (R_len_t k=0; k < patternLen; ++k)
+         if (searchStr[byteindex+k] != patternStr[k])
+            return false;
+   }
+
+   return true; // found
+}
