@@ -43,6 +43,7 @@ StriContainerRegexPattern::StriContainerRegexPattern()
 {
    this->lastMatcherIndex = -1;
    this->lastMatcher = NULL;
+   this->flags =0;
 }
 
 
@@ -117,11 +118,8 @@ RegexMatcher* StriContainerRegexPattern::getMatcher(R_len_t i)
 
    UErrorCode status = U_ZERO_ERROR;
    lastMatcher = new RegexMatcher(this->get(i), flags, status);
-   if (U_FAILURE(status)) {
-      delete lastMatcher;
-      lastMatcher = NULL;
-      throw StriException(status);
-   }
+   STRI__CHECKICUSTATUS_THROW(status, {if (lastMatcher) delete lastMatcher; lastMatcher = NULL;})
+   if (!lastMatcher) throw StriException(MSG__MEM_ALLOC_ERROR);
    this->lastMatcherIndex = (i % n);
 
    return lastMatcher;
