@@ -41,8 +41,9 @@
 
 #define STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont, naset, zeroset)        \
       if ((str_cont).isNA(i) || (pattern_cont).isNA(i) || (pattern_cont).get(i).length() <= 0) { \
-         if (!(pattern_cont).isNA(i) && (pattern_cont).get(i).length() <= 0)                     \
+         if ((!(pattern_cont).isNA(i)) && (pattern_cont).get(i).length() <= 0) {                 \
             Rf_warning(MSG__EMPTY_SEARCH_PATTERN_UNSUPPORTED);                                   \
+         }                                                                                       \
          naset;                                                                                  \
          continue;                                                                               \
       }                                                                                          \
