@@ -48,7 +48,7 @@ using namespace std;
  *
  * @param str character vector
  * @param pattern character vector
- * @param n_max integer vector
+ * @param n integer vector
  * @param omit_empty logical vector
  * @param opts_collator passed to stri__ucol_open(),
  * if \code{NA}, then \code{stri_detect_fixed_byte} is called
@@ -80,26 +80,29 @@ using namespace std;
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-04)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-04)
+ *    allow `simplify=NA`; FR #126: pass n to stri_list2matrix
  */
-SEXP stri_split_coll(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty,
+SEXP stri_split_coll(SEXP str, SEXP pattern, SEXP n, SEXP omit_empty,
                      SEXP tokens_only, SEXP simplify, SEXP opts_collator)
 {
+   bool tokens_only1 = stri__prepare_arg_logical_1_notNA(tokens_only, "tokens_only");
    PROTECT(str = stri_prepare_arg_string(str, "str"));
    PROTECT(pattern = stri_prepare_arg_string(pattern, "pattern"));
-   PROTECT(n_max = stri_prepare_arg_integer(n_max, "n_max"));
+   PROTECT(n = stri_prepare_arg_integer(n, "n"));
    PROTECT(omit_empty = stri_prepare_arg_logical(omit_empty, "omit_empty"));
-   bool tokens_only1 = stri__prepare_arg_logical_1_notNA(tokens_only, "tokens_only");
-   bool simplify1 = stri__prepare_arg_logical_1_notNA(simplify, "simplify");
+   PROTECT(simplify = stri_prepare_arg_logical_1(simplify, "simplify"));
 
    UCollator* collator = NULL;
    collator = stri__ucol_open(opts_collator);
 
-   STRI__ERROR_HANDLER_BEGIN(4)
+   STRI__ERROR_HANDLER_BEGIN(5)
    R_len_t vectorize_length = stri__recycling_rule(true, 4,
-      LENGTH(str), LENGTH(pattern), LENGTH(n_max), LENGTH(omit_empty));
+      LENGTH(str), LENGTH(pattern), LENGTH(n), LENGTH(omit_empty));
    StriContainerUTF16 str_cont(str, vectorize_length);
    StriContainerUStringSearch pattern_cont(pattern, vectorize_length, collator);  // collator is not owned by pattern_cont
-   StriContainerInteger   n_max_cont(n_max, vectorize_length);
+   StriContainerInteger   n_cont(n, vectorize_length);
    StriContainerLogical   omit_empty_cont(omit_empty, vectorize_length);
 
    SEXP ret;
@@ -109,60 +112,60 @@ SEXP stri_split_coll(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty,
          i != pattern_cont.vectorize_end();
          i = pattern_cont.vectorize_next(i))
    {
-      if (n_max_cont.isNA(i)) {
+      if (n_cont.isNA(i)) {
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));
          continue;
       }
 
-      int  n_max_cur        = n_max_cont.get(i);
+      int  n_cur        = n_cont.get(i);
       int  omit_empty_cur   = !omit_empty_cont.isNA(i) && omit_empty_cont.get(i);
 
       STRI__CONTINUE_ON_EMPTY_OR_NA_STR_PATTERN(str_cont, pattern_cont,
          SET_VECTOR_ELT(ret, i, stri__vector_NA_strings(1));,
          SET_VECTOR_ELT(ret, i,
             (omit_empty_cont.isNA(i))?stri__vector_NA_strings(1):
-            stri__vector_empty_strings((omit_empty_cur || n_max_cur == 0)?0:1));)
+            stri__vector_empty_strings((omit_empty_cur || n_cur == 0)?0:1));)
 
       UStringSearch *matcher = pattern_cont.getMatcher(i, str_cont.get(i));
       usearch_reset(matcher);
 
 
-      if (n_max_cur >= INT_MAX-1)
-         throw StriException(MSG__EXPECTED_SMALLER, "n_max");
-      else if (n_max_cur < 0)
-         n_max_cur = INT_MAX;
-      else if (n_max_cur == 0) {
+      if (n_cur >= INT_MAX-1)
+         throw StriException(MSG__EXPECTED_SMALLER, "n");
+      else if (n_cur < 0)
+         n_cur = INT_MAX;
+      else if (n_cur == 0) {
          SET_VECTOR_ELT(ret, i, Rf_allocVector(STRSXP, 0));
          continue;
       }
       else if (tokens_only1)
-         n_max_cur++; // we need to do one split ahead here
+         n_cur++; // we need to do one split ahead here
 
       R_len_t k;
       deque< pair<R_len_t, R_len_t> > fields; // byte based-indices
       fields.push_back(pair<R_len_t, R_len_t>(0,0));
       UErrorCode status = U_ZERO_ERROR;
 
-      for (k=1; k < n_max_cur && USEARCH_DONE != usearch_next(matcher, &status) && !U_FAILURE(status); ) {
+      for (k=1; k < n_cur && USEARCH_DONE != usearch_next(matcher, &status) && !U_FAILURE(status); ) {
          R_len_t s1 = (R_len_t)usearch_getMatchedStart(matcher);
          R_len_t s2 = (R_len_t)usearch_getMatchedLength(matcher) + s1;
 
          if (omit_empty_cur && fields.back().first == s1)
-            fields.back().first = s2; // don't start new field
+            fields.back().first = s2; // don't start any new field
          else {
             fields.back().second = s1;
-            fields.push_back(pair<R_len_t, R_len_t>(s2, s2)); // start new field here
+            fields.push_back(pair<R_len_t, R_len_t>(s2, s2)); // start a new field here
             ++k; // another field
          }
       }
-      if (U_FAILURE(status)) throw StriException(status);
+      STRI__CHECKICUSTATUS_THROW(status, {/* do nothing special on err */})
       fields.back().second = str_cont.get(i).length();
       if (omit_empty_cur && fields.back().first == fields.back().second)
          fields.pop_back();
 
-      if (tokens_only1 && n_max_cur < INT_MAX) {
-         n_max_cur--; // one split ahead could have been made, see above
-         while (fields.size() > (size_t)n_max_cur)
+      if (tokens_only1 && n_cur < INT_MAX) {
+         n_cur--; // one split ahead could have been made, see above
+         while (fields.size() > (size_t)n_cur)
             fields.pop_back(); // get rid of the remainder
       }
 
@@ -182,9 +185,18 @@ SEXP stri_split_coll(SEXP str, SEXP pattern, SEXP n_max, SEXP omit_empty,
 
    if (collator) { ucol_close(collator); collator=NULL; }
 
-   if (simplify1) {
-      ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
-         stri__vector_NA_strings(1));
+   if (LOGICAL(simplify)[0] == NA_LOGICAL || LOGICAL(simplify)[0]) {
+      R_len_t n_min = 0;
+      R_len_t n_length = LENGTH(n);
+      int* n_tab = INTEGER(n);
+      for (R_len_t i=0; i<n_length; ++i) {
+         if (n_tab[i] != NA_INTEGER && n_min < n_tab[i])
+            n_min = n_tab[i];
+      }
+      STRI__PROTECT(ret = stri_list2matrix(ret, Rf_ScalarLogical(TRUE),
+         (LOGICAL(simplify)[0] == NA_LOGICAL)?stri__vector_NA_strings(1)
+                                             :stri__vector_empty_strings(1),
+         Rf_ScalarInteger(n_min)))
    }
 
    STRI__UNPROTECT_ALL
