@@ -65,9 +65,13 @@ using namespace std;
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    FR #110, #23: opts_fixed arg added
  */
-SEXP stri__replace_allfirstlast_fixed(SEXP str, SEXP pattern, SEXP replacement, int type)
+SEXP stri__replace_allfirstlast_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_fixed, int type)
 {
+   uint32_t pattern_flags = StriContainerByteSearch::getByteSearchFlags(opts_fixed);
    PROTECT(str          = stri_prepare_arg_string(str, "str"));
    PROTECT(pattern      = stri_prepare_arg_string(pattern, "pattern"));
    PROTECT(replacement  = stri_prepare_arg_string(replacement, "replacement"));
@@ -76,7 +80,7 @@ SEXP stri__replace_allfirstlast_fixed(SEXP str, SEXP pattern, SEXP replacement,
    STRI__ERROR_HANDLER_BEGIN(3)
    StriContainerUTF8 str_cont(str, vectorize_length);
    StriContainerUTF8 replacement_cont(replacement, vectorize_length);
-   StriContainerByteSearch pattern_cont(pattern, vectorize_length);
+   StriContainerByteSearch pattern_cont(pattern, vectorize_length, pattern_flags);
 
    SEXP ret;
    STRI__PROTECT(ret = Rf_allocVector(STRSXP, vectorize_length));
@@ -233,8 +237,11 @@ SEXP stri__replace_allfirstlast_fixed(SEXP str, SEXP pattern, SEXP replacement,
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-05)
  *    Issue #112: str_prepare_arg* retvals were not PROTECTed from gc
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    FR #110, #23: opts_fixed arg added
  */
-SEXP stri__replace_all_fixed_no_vectorize_all(SEXP str, SEXP pattern, SEXP replacement)
+SEXP stri__replace_all_fixed_no_vectorize_all(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_fixed)
 { // version gamma:
    PROTECT(str          = stri_prepare_arg_string(str, "str"));
 
@@ -259,15 +266,17 @@ SEXP stri__replace_all_fixed_no_vectorize_all(SEXP str, SEXP pattern, SEXP repla
 
    if (pattern_n == 1) { // this will be much faster:
       SEXP ret;
-      PROTECT(ret = stri__replace_allfirstlast_fixed(str, pattern, replacement, 0));
+      PROTECT(ret = stri__replace_allfirstlast_fixed(str, pattern, replacement, opts_fixed, 0));
       UNPROTECT(4);
       return ret;
    }
 
+   uint32_t pattern_flags = StriContainerByteSearch::getByteSearchFlags(opts_fixed);
+
    STRI__ERROR_HANDLER_BEGIN(3)
    StriContainerUTF8 str_cont(str, str_n, false); // writable
    StriContainerUTF8 replacement_cont(replacement, pattern_n);
-   StriContainerByteSearch pattern_cont(pattern, pattern_n);
+   StriContainerByteSearch pattern_cont(pattern, pattern_n, pattern_flags);
 
    for (R_len_t i = 0; i<pattern_n; ++i)
    {
@@ -510,13 +519,16 @@ SEXP stri__replace_all_fixed_no_vectorize_all(SEXP str, SEXP pattern, SEXP repla
  *
  * @version 0.3-1 (Marek Gagolewski, 2014-11-01)
  *          vectorize_all argument added
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    FR #110, #23: opts_fixed arg added
  */
-SEXP stri_replace_all_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP vectorize_all)
+SEXP stri_replace_all_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP vectorize_all, SEXP opts_fixed)
 {
    if (stri__prepare_arg_logical_1_notNA(vectorize_all, "vectorize_all"))
-      return stri__replace_allfirstlast_fixed(str, pattern, replacement, 0);
+      return stri__replace_allfirstlast_fixed(str, pattern, replacement, opts_fixed, 0);
    else
-      return stri__replace_all_fixed_no_vectorize_all(str, pattern, replacement);
+      return stri__replace_all_fixed_no_vectorize_all(str, pattern, replacement, opts_fixed);
 }
 
 
@@ -535,10 +547,13 @@ SEXP stri_replace_all_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP vecto
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-08)
  *          stri_replace_fixed now uses byte search only
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    FR #110, #23: opts_fixed arg added
  */
-SEXP stri_replace_last_fixed(SEXP str, SEXP pattern, SEXP replacement)
+SEXP stri_replace_last_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_fixed)
 {
-   return stri__replace_allfirstlast_fixed(str, pattern, replacement, -1);
+   return stri__replace_allfirstlast_fixed(str, pattern, replacement, opts_fixed, -1);
 }
 
 
@@ -557,8 +572,11 @@ SEXP stri_replace_last_fixed(SEXP str, SEXP pattern, SEXP replacement)
  *
  * @version 0.2-3 (Marek Gagolewski, 2014-05-08)
  *          stri_replace_fixed now uses byte search only
+ *
+ * @version 0.4-1 (Marek Gagolewski, 2014-12-07)
+ *    FR #110, #23: opts_fixed arg added
  */
-SEXP stri_replace_first_fixed(SEXP str, SEXP pattern, SEXP replacement)
+SEXP stri_replace_first_fixed(SEXP str, SEXP pattern, SEXP replacement, SEXP opts_fixed)
 {
-   return stri__replace_allfirstlast_fixed(str, pattern, replacement, 1);
+   return stri__replace_allfirstlast_fixed(str, pattern, replacement, opts_fixed, 1);
 }
